<?php
/**
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   LGPL 3.0
 *
 * Stand:    7.10.2021
 *  
 * Absicherung gegen WEB-Angriffe. Wird am Anfang jeder Seite aufgerufen.
 * Enthält auch Funktionen zur späteren Prüfung.
 */
require_once './includes/classes.inc';
if (session_status() == PHP_SESSION_NONE)  session_start();
/**
 * Erzeugt das PDO-Objekt $dbh für den Zugriff auf die Datenbank.
 */
require_once './phpsec/createDBH.inc';

/**
 * Trägt eine IP-Adresse in die Tabelle der gesperrten Adressen ein.
 * error_count wird auf 4 gesetzt, damit die Sperrung sofort wirkt.
 * @param String(16) $grund: Grund der Sperrung
 */
function defenderSperrt($grund){
    global $dbh;
    if (strlen($grund) > 16) $grund = substr($grund, 0,16);
    $stmt = $dbh->prepare('replace wsrDefender set IP_adresse = :IP, error_count = 4, datumUhrzeit = now(), error_type=:grund');
    defenderCheck($stmt, 'Prepare Sperren');
    $erg = $stmt->execute(array('IP' => $_SERVER['REMOTE_ADDR'], 'grund' => $grund));
    defenderCheck($erg, 'Execute Sperren');
    die();
}

/**
 * Trägt eine IP-Adresse in die Tabelle der gesperrten Adressen ein, bzw.
 * erhöht in einem bereits vorhandenen Eingetrag error_count um 1.
 * Die Sperrung wirkt dann erst nach dem 3. Aufruf.
 */
function defenderUngueltigerUser(){
    global $dbh;
    $stmt = $dbh->prepare('select error_count from wsrDefender where IP_adresse = :IP ');
    defenderCheck($stmt, 'Prepare Select für ungültigen User');
    $erg = $stmt->execute(array('IP' => $_SERVER['REMOTE_ADDR']));
    defenderCheck($erg, 'Execute Select für ungültigen User');
    if ($z = $stmt->fetchObject()){
        $ec = $z->error_count + 1;
    }else{
        $ec = 1;
    }
    $stmt = $dbh->prepare('replace wsrDefender set IP_adresse = :IP, error_count = :ec, '
        .'datumUhrzeit = now(), error_type=:grund, webRequest=:user');
    defenderCheck($stmt, 'Prepare Eintrag für ungültigen User');
    $erg = $stmt->execute(array('IP' => $_SERVER['REMOTE_ADDR'], 'grund' => 'User ungültig', 'ec' => $ec
        , 'user'=>$_REQUEST['user'].' / '.$_REQUEST['pass'] ));
    defenderCheck($erg, 'Execute Eintrag für ungültigen User');
}

/**
 * Ergebnis der Datenbank-Transaktion prüfen und 
 * ggf. Fehlermeldung mailen und Verarbeitung abbrechen.
 */
function defenderCheck($ergebnis,$meldung){
    global $dbh;
    if ($ergebnis === false){
        mail('wisch@ws-unternehmensberatung.com','WS Defender Fehlermeldung',
            'Im Server ist bei einer Datenbank-Transaktion der folgende Fehler aufgetreten'
            ."\nDBH: ".$dbh->errorInfo()[2]
            ."\nMeldung:\n".$meldung,
            "From: ws-defender@ws-unternehmensberatung.com\n"
            );
        die();
    }
}




/**
 * Prüfen, ob IP-Adresse gesperrt ist.
 */
$stmt = $dbh->prepare('select count(*) as count from wsrDefender where IP_adresse = :IP and  error_count > 3');
defenderCheck($stmt, 'Prepare Select für Start IP-Adresse');
$erg = $stmt->execute(array('IP' => $_SERVER['REMOTE_ADDR']));
defenderCheck($erg, 'Execute Select für Start IP-Adresse');
$c = $stmt->fetchObject()->count;
if ($c > 0){
    echo 'Die IP-Adresse '.$_SERVER['REMOTE_ADDR'].' ist gesperrt.<br>Bitte wenden Sie sich an den Systemadministrator.';
    die();
}


/**
 * Prüfen, ob das Session-Token passt.
 */
if (array_key_exists('Token', $_SESSION)){
    $token = $_SESSION['Token'];
    if (array_key_exists('WS-Report-Token', $_COOKIE)){
        if ($token != $_COOKIE['WS-Report-Token']){
            http_response_code(425);
            echo "FEHLER: Das Anmelde-Token passt nicht.<br>Die Anwendung wird gesperrt.";
            defenderSperrt('Token ungültig');
        }
    }else{
        echo "FEHLER: Das Anmelde-Token fehlt.<br>Die Anwendung wird gesperrt.";
        defenderSperrt('Token fehlt');
    }
}

/**
 * Bei versuchter Anmeldung user und Passwort pass prüfen
 * und ggf. Fehler eintragen.
 */
global $editUser;
if (array_key_exists('user', $_REQUEST) && ! isset($editUser)){
    $stmt = $dbh->prepare('select error_count, password from wsrUser where user = :user');
    defenderCheck($stmt, 'Prepare Select für Start User-Anmeldung');
    $erg = $stmt->execute(array('user' => $_REQUEST['user']));
    defenderCheck($erg, 'Execute Select für Start User-Anmeldung');
    if ($z = $stmt->fetchObject()){
        if ($z->error_count > 3){
            echo 'Der Benutzer '.$_REQUEST['user'].' ist gesperrt.<br>Bitte wenden Sie sich an den Systemadministrator.';
            die();
        }else 
            if ($z->password != null 
                && strlen(trim($z->password)) > 0
                && trim($z->password) != trim($_REQUEST['pass'])
                ) {
                    $stmt = $dbh->prepare('update wsrUser set error_count = :ec where user = :user');
                    defenderCheck($stmt, 'Prepare Update für User-Anmeldung');
                    $erg = $stmt->execute(array('ec' => $z->error_count + 1, 'user' => $_REQUEST['user']));
                    defenderCheck($erg, 'Execute Update für User-Anmeldung');
                    if ($z->error_count > 2) {
                        echo "FEHLER: 3 * User / Passport falsch.<br>Die Anwendung wird gesperrt.";                        
                        defenderSperrt('Passwort falsch');                    
                    }
                }
        else{
            // Eine gültige Anmeldung löscht eventuell vorhandene Fehlereinträge
            $stmt = $dbh->prepare('update wsrUser set error_count = 0 where user = :user');
            defenderCheck($stmt, 'Prepare Bereinigen der User-Anmeldung');
            $erg = $stmt->execute(array('user' => $_REQUEST['user']));
            defenderCheck($erg, 'Execute Bereinigen der User-Anmeldung');
            $stmt = $dbh->prepare('delete from wsrDefender  where IP_adresse = :IP');
            defenderCheck($stmt, 'Prepare Bereinigen des Fehler-Zählers');
            $erg = $stmt->execute(array('IP' => $_SERVER['REMOTE_ADDR']));
            defenderCheck($erg, 'Execute Bereinigen des Fehler-Zählers');
        }
    }else{
        defenderUngueltigerUser();
    }
}

/**
 * Die Session wird hier geschlossen, weil auch Seiten abgesichert werden,
 * die die Session erst später oder gar nicht einlesen.
 */
if (session_status() == PHP_SESSION_ACTIVE) session_abort();


