<?php
/**
 * Projekt: WS-Reporter
 *          Reportdefinition einlesen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    25.06.2022
 * 
 * Status: Neue Version in Arbeit
 *
 */
require_once './includes/loadClassDef.inc';
require_once './includes/funktionen.inc';
require_once './phpsec/createDBH.inc';

// error_log(print_r($_REQUEST,true));

function loadReport($title){
    global $query, $meldungen, $dbh;
    $repTitleVar = 'reportTitle_de';
    $sprache = 'de';
    if (array_key_exists('Sprache', $_SESSION))
        switch (substr($_SESSION['Sprache'],0,2)){
            case 'de': $repTitleVar = 'reportTitle_de';
            break;
            case 'en': $repTitleVar = 'reportTitle_en';
            $sprache = 'en';
            break;
            case 'fr': $repTitleVar = 'reportTitle_fr';
            $sprache = 'fr';            
            break;
    }
    $sql = new SqlAbfrage('WSR');
    $sql->sql = "select reportID, reportVersion from reportTitle";
    $sql->addSel($repTitleVar." = ", $title);
    $stmt = $sql->execute();
    if ($stmt === false){
        http_response_code(425);
        echo 'Request:<br><pre>';
        echo var_dump($_REQUEST);
        echo '</pre><br>DB-Fehler:<br><pre>';
        echo var_dump($sql->errorInfo());
        echo '</pre>';
        die();
    }
    $z = $stmt->fetchObject();
    if ($z === false){
        return false;
    }
    return loadReportID($z->reportID);
}

function loadReportID($reportID){
    global $query, $meldungen, $dbh;
    $repTitleVar = 'reportTitle_de';
    $sprache = 'de';
    if (array_key_exists('Sprache', $_SESSION))
        switch (substr($_SESSION['Sprache'],0,2)){
            case 'de': $repTitleVar = 'reportTitle_de';
            break;
            case 'en': $repTitleVar = 'reportTitle_en';
            $sprache = 'en';
            break;
            case 'fr': $repTitleVar = 'reportTitle_fr';
            $sprache = 'fr';
            break;
    }
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select '.$repTitleVar.' as title, reportVersion from reportTitle';
    $sql->addSel("reportID = ", $reportID);
    $stmt = $sql->execute();
    if ($stmt === false){
        http_response_code(425);
        echo 'Request:<br><pre>';
        echo var_dump($_REQUEST);
        echo '</pre><br>DB-Fehler:<br><pre>';
        echo var_dump($sql->errorInfo());
        echo '</pre>';
        die();
    }
    $z = $stmt->fetchObject();
    if ($z === false){
        return false;
    }
    $title = $z->title;
    if ($z->reportVersion == 1){
        require_once './includes/loadReportDefV1.inc';
        $erg = loadReportV1($title);
        if ($erg) $query->reportID = $reportID;
        return $erg;
    }
    
    error_log ('Lese neue Report-Version');
    
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select * from reportAttributes ';
    $sql->addSel('reportID = ', $reportID);
    $stmt = $sql->execute();
    if ($stmt === false){
        echo "FEHLER beim Lesen von reportAttributes:\n";
        echo $sql->errorInfo()[2];
        die();
    }
    $z = $stmt->fetchObject();
    if ($z === false){
        echo "FEHLER beim Lesen von reportAttributes:\n";
        echo "ReportID $reportID nicht gefnden\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    $query = new Query($z->table_name);
    if ((array_key_exists('action', $_REQUEST) && $_REQUEST['action'] == 'reload')) {
        $query->loadSaveAddFilter = true;
    }
    loadClassDef($query);
    $query->newReport();

    if (isValid($z->rLanguage)) $query->rLanguage = $z->rLanguage;
    else  $query->rLanguage = 'de';
    $rTranslations = explode(' ',trim($z->rTranslations));
//    error_log('>'.$z->rTranslations.'<');
//    error_log(print_r($rTranslations,true));
    $query->rTranslations = $rTranslations;
    if ($query->currentLanguage == $query->rLanguage){
        $query->currentRLanguage = $query->rLanguage;
    }else if (false === array_search($query->currentLanguage, $rTranslations)){
        $meldungen.="\n<br>ACHTUNG: Für die aktuelle Sprache ".$query->currentLanguage
        .' gibt es keine Übersetzung des Reports.';
        $query->currentRLanguage = $query->rLanguage;
        error_log($meldungen);
        // Spracheinstellung des Reports hier nicht überschreiben ???
        //    }else{
        //        $query->rTranslations[] = $query->rLanguage;
        //        $query->rLanguage = $query->currentLanguage;
        }else{
            $query->currentRLanguage = $query->currentLanguage;
        }
        error_log('Einzulesender Report wird von '.$query->rLanguage.' nach '.$query->currentRLanguage.' übersetzt.');
    $query->reportID = $reportID;
    $query->title = $title;
//    error_log('Vorher:'.print_r($query->repFilter,true));    

// Nicht-Hidden Filter werden hier entfernt und später bei createReport berücksichtigt (??)
    $rfv = $query->repFilter;
    for($i = $rfv->size(); $i > 0; ){
        $i --;
        $rf = $rfv->getPos($i);
        if (! $query->metaData[$rf->getName()]->hidden) $rfv->removePos($i);
    }
    
    // Attribute übernehmen
    $query->sort_mode = $z->sort_mode;
    $query->repGeldFaktor = $z->repGeldFaktor;
    $query->fix_labelColumns = $z->fix_labelColumns;
    $query->values_in_columns = $z->values_in_columns;
    $query->folderMode = $z->folderMode;
    $query->valueColumnFolderMode = $z->valueColumnFolderMode;
    $query->chart_size = $z->chart_size;
    $query->display_charts = $z->display_charts;
    $query->dropSummen = $z->dropSummen;
    $historyOptions = $z->historyOptions;
    $histEntries = explode(':', $historyOptions);
//            error_log(print_r($histEntries,true));
    foreach ($histEntries as $hEntry){
        $hEntryArray = explode('.',$hEntry);
//                    error_log(print_r($hEntryArray,true));
        if (isValid($hEntryArray[0])){
            if (isValid($hEntryArray[1])){
                if ($hEntryArray[0] == 0){
                    $query->historyOption = 1* $hEntryArray[1];
                    if (array_key_exists(2, $hEntryArray)){
                        $query->historyDate = $hEntryArray[2];
                    }
                }else{
                    /**
                     * 
                     * @var Dimension $dim
                     */
                    $dim = $query->dimensions[$hEntryArray[0]];
                    if ($dim == null){
//                        error_log("Fehler beim Einlesen der Historisierung Nr {$hEntryArray[0]} für die Basisquelle ".$historyOptions);
                    }else {
                        $dim->historyOption = 1* $hEntryArray[1];
                        if (array_key_exists(2, $hEntryArray)){
                            $dim->historyDate = $hEntryArray[2];
                        }
//                        error_log("ergibt Dim Nr. {$hEntryArray[0]} ".$dim->dimName.' - '. $hEntryArray[1]);
                    }
                }
            }
        }
    }
    
    // Verarbeitung erfolgt unten
    $query->repWerteBasis = $z->repWerteBasis;
    $query->repWerteAuswahlTitel = $z->repWerteAuswahlTitel;
    $query->repWerteAuswahlLabel = $z->repWerteAuswahlLabel;
    $query->repWerteAuswahlSelector = $z->repWerteAuswahlSelector;
    
    // Joined Tables einlesen
    $stmt = $dbh->prepare(JoinedTable::$readStatement);
    if ($stmt === false){
        echo "FEHLER bei prepare READ JoinedTable\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von JoinedTable\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    while ($z = $stmt->fetchObject()) {
        
//        error_log('Verknüpfte Datenquelle '.$z->table_name);
        $aq = $query->addQueries->get($z->table_name);
        if ($aq == null){
            $meldungen .= "\n<br>Fehler: Der Report verlangt die verknüpfte Datenquelle <b>".$z->table_name
            .'</b> als <b>'.$z->kennzeichen.'</b> ('.$z->prefix.').<br>Sie ist in der Basis-Datenquelle nicht deklariert.';
            //                        return;
        }else{
            $jt = new JoinedTable($aq);
            $jt->read($z);
            if ($jt->query == null || $jt->query->phys_table == null){
                $meldungen .= "\n<br>Fehler beim Lesen der verknüpften Datenquelle <b>".$z->table_name
                .'</b> als <b>'.$z->kennzeichen.'</b> ('.$z->prefix.')';
            }
            $jt->prefix = qTranslate::translateReport($query, $jt->prefix);
            $query->repTables->put($jt);
//            error_log('Verknüpfte Datenquelle hat Kennzeichen '.$jt->kennzeichen);
        }
    }
    $stmt->closeCursor();
    
    
    
    // Rows einlesen
    $stmt = $dbh->prepare(RepRow::$readStatement);
    if ($stmt === false){
        echo "FEHLER bei prepare READ RepRow\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von RepRow\n";
        echo $stmt->errorInfo()[2];
        die();      
    }
    while ($z = $stmt->fetchObject()) {
        $repRow = new RepRow($z->column_name);
        $repRow->read($z);        
        if (isValid($repRow->web_title)) $repRow->web_title = qTranslate::translateReport($query, $repRow->web_title);
        if (isValid($repRow->web_link_title)) $repRow->web_link_title = qTranslate::translateReport($query, $repRow->web_link_title);
        if (isValid($repRow->web_link_value)) $repRow->web_link_value = qTranslate::translateReport($query, $repRow->web_link_value);
        if (isValid($repRow->web_link)) $repRow->web_link = qTranslate::translateWebLink($query, $repRow->web_link);
        $query->repRows->put($repRow);
    }
    $stmt->closeCursor();
    
    // Values einlesen
    $stmt = $dbh->prepare(RepValue::$readStatement);
    if ($stmt === false){
        echo "FEHLER bei prepare READ RepValue\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von RepValue\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    while ($z = $stmt->fetchObject()) {
        $repValue = new RepValue($z->column_name);
        $repValue->read($z);
        if (isValid($repValue->label)) $repValue->label = qTranslate::translateReport($query, $repValue->label);        
        $query->repValues->put($repValue);
        if (substr($repValue->getValueName(), 1,1) == ':' 
            || substr($repValue->getValueName(), 1,1) == '=' ){
            $k = substr($repValue->getValueName(), 0,1);
            $rt = $query->repTables->get($k);
            if ($rt == null){
                echo 'FEHLER Verknüpfte Datenquelle '.$k.' ist nicht spezifiziert.<br>';
            }
        }
    }
    $stmt->closeCursor();
    
    // Columns einlesen
    $stmt = $dbh->prepare(RepColumn::$readStatement);
    if ($stmt === false){
        echo "FEHLER bei prepare READ RepColumn\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von RepColumn\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    while ($z = $stmt->fetchObject()) {
        $repColumn = new RepColumn($z->column_name);
        $repColumn->read($z);
        if (isValid($repColumn->label)) $repColumn->label = qTranslate::translateReport($query, $repColumn->label);
        if (isValid($repColumn->web_link)) $repColumn->web_link = qTranslate::translateWebLink($query, $repColumn->web_link);
        if (isValid($repColumn->web_title)) $repColumn->web_title = qTranslate::translateReport($query, $repColumn->web_title);
        
        if (isValid($repColumn->filter)){
            $mr = $query->getReportMetaRecord($repColumn->filter);
            if ($mr == NULL
                && $mr->filterType == MetaRecord::SONDERFILTERSEL
                && isValid($repColumn->fs_von)) 
                    $repColumn->fs_von = qTranslate::translateReport($query, $repColumn->fs_von);
        }        
        $query->repColumns->put($repColumn);
        if (substr($repColumn->getName(), 1,1) == ':'
            || substr($repColumn->getName(), 1,1) == '=' ){
                $k = substr($repColumn->getName(), 0,1);
                $rt = $query->repTables->get($k);
                if ($rt == null){
                    echo 'FEHLER Verknüpfte Datenquelle '.$k.' ist nicht spezifiziert.<br>';
                }
        }
    }
    $stmt->closeCursor();
       
//    error_log('Nachher:'.print_r($query->repFilter,true));
    // Filter einlesen
    $stmt = $dbh->prepare(RepFilter::$readStatement);
    if ($stmt === false){
        echo "FEHLER bei prepare READ RepFilter\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von RepFilter\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    while ($z = $stmt->fetchObject()) {
        $repFilter = new RepFilter($z->column_name);
        $repFilter->read($z);
        $mr = $query->getReportMetaRecord($repFilter->getName());
        if ($mr != null && $mr->filterType == MetaRecord::SONDERFILTERSEL){
            $repFilter->fs_von = qTranslate::translateReport($query, $repFilter->fs_von);
        }        
        $query->repFilter->put($repFilter);
        if (substr($repFilter->getName(), 1,1) == ':'
            || substr($repFilter->getName(), 1,1) == '=' ){
                $k = substr($repFilter->getName(), 0,1);
                $rt = $query->repTables->get($k);
                if ($rt == null){
                    echo 'FEHLER Verknüpfte Datenquelle '.$k.' ist nicht spezifiziert.<br>';
                }
        }
        $mr = $query->getReportMetaRecord($z->column_name);
        if ($mr != null && $mr->filterType == MetaRecord::LIST){
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'select * from reportFilterLists';
            $sql->addSel('reportID = ', $query->reportID);
            $sql->addSel('column_name = ', $z->column_name);
            $stmtList = $sql->execute();
            if (false === $stmtList){
                echo "FEHLER bem Lesen von RepFilterList\n";
                echo $sql->errorInfo()[2];
                die();
            }
            while ($zList = $stmtList->fetchObject()){
                $repFilter->fs_list[] = $zList->fs_von;
            }
            $stmtList->closeCursor();
        }
    }
    $stmt->closeCursor();
    
    
    //  AddFilter einlesen
    $stmt = $dbh->prepare(str_replace('reportFilters', 'reportAddFilters',RepFilter::$readStatement));
    if ($stmt === false){
        echo "FEHLER bei prepare READ RepAddFilter\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von RepAddFilter\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    while ($z = $stmt->fetchObject()) {
        $repFilter = new RepFilter($z->column_name);
        $repFilter->read($z);
        $mr = $query->getReportMetaRecord($repFilter->getName());
        if ($mr != null && $mr->filterType == MetaRecord::SONDERFILTERSEL){
            $repFilter->fs_von = qTranslate::translateReport($query, $repFilter->fs_von);
        }
        $query->addFilter->put($repFilter);
        if (substr($repFilter->getName(), 1,1) == ':'
            || substr($repFilter->getName(), 1,1) == '=' ){
                $k = substr($repFilter->getName(), 0,1);
                $rt = $query->repTables->get($k);
                if ($rt == null){
                    echo 'FEHLER Verknüpfte Datenquelle '.$k.' ist nicht spezifiziert.<br>';
                }
        }
        $mr = $query->getReportMetaRecord($z->column_name);
        if ($mr != null && $mr->filterType == MetaRecord::LIST){
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'select * from reportAddFilterLists';
            $sql->addSel('reportID = ', $query->reportID);
            $sql->addSel('column_name = ', $z->column_name);
            $stmtList = $sql->execute();
            if (false === $stmtList){
                echo "FEHLER bem Lesen von RepFilterList\n";
                echo $sql->errorInfo()[2];
                die();
            }
            while ($zList = $stmtList->fetchObject()){
                $repFilter->fs_list[] = $zList->fs_von;
            }
            $stmtList->closeCursor();
        }
    }
    $stmt->closeCursor();
    
    
    // repReports einlesen
    // Hier sollte auf die Report-ID umgestellt werden?
    $stmt = $dbh->prepare(RepReport::$readStatement);
    if ($stmt === false){
        echo "FEHLER bei prepare READ RepReport\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    if (false === $stmt->execute(array ('reportID' => $reportID))){
        echo "FEHLER bem Lesen von RepReport\n";
        echo $stmt->errorInfo()[2];
        die();
    }
    while ($z = $stmt->fetchObject()) {
        $repReport = new RepReport(qTranslate::translateReportTitle($z->reportTitle));
        $repReport->read($z,$query);
        $query->repReports->put($repReport);
    }
    $stmt->closeCursor();
    
    // Kommentar einlesen
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select * from reportComment';
    $sql->addSel('reportID = ', $query->reportID);
    $sql->addSel('rLanguage = ', $query->currentRLanguage);
    $sql->sql .= "\n order by SortNr";
    $stmt = $sql->execute();
    if ($stmt === false){
        echo "FEHLER bei READ RepComment\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    $query->kommentar = '';
    while ($z = $stmt->fetchObject()) {
        $query->kommentar .= $z->kommentar;
    }
    $stmt->closeCursor();

    // Technische Hinweise einlesen
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select * from reportHints';
    $sql->addSel('reportID = ', $query->reportID);
    $sql->addSel('rLanguage = ', $query->currentRLanguage);
    $sql->sql .= "\n order by SortNr";
    $stmt = $sql->execute();
    if ($stmt === false){
        echo "FEHLER bei READ RepHints\n";
        echo $dbh->errorInfo()[2];
        die();
    }
    $query->report_hints = '';
    while ($z = $stmt->fetchObject()) {
        $query->report_hints .= $z->kommentar;
    }
    $stmt->closeCursor();
    
    
    $query->titleModified = false;
    rechteAnwenden();
    return true;
}