<?php
/**
 * Projekt: WS-Reporter
 *          Werte für Datenerfassung eintragen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    04.03.2023
 * 
 * ACHTUNG: Das Update funktioniert nicht bei verknüpften Quellen!
 *          Nur deutsche Zahlen-Formate !!!
 *
 */

include 'includes/defender.inc';
global $query, $sql, $rows;
require_once './phpsec/createDBH.inc';
require_once './includes/createReport.inc';
require_once './includes/classes.inc';
require_once './includes/funktionen.inc';

function addSel($sql){
    global $query,  $rows;
    $rowCount = 0;    
    for ($i=0; $i < $query->repRows->size(); $i++){
        if (!$query->repRows->getPos($i)->webLinkColumn) $rowCount ++;
    }
    error_log(count($rows)." Zeilenwerte für $rowCount Zeilen");
    $rowCount = 0;
    for ($i=0; $i < $query->repRows->size(); $i++){
        if ($query->repRows->getPos($i)->webLinkColumn) continue;
        if ($rowCount >= count($rows)){
            error_log('updateValue: Es wurden nur '.count($rows).' Zeilenwerte übergeben. Definiert sind '.$query->repRows->size().' Zeilen');
            $i = $query->repRows->size();
            continue;
        }
        if (!isValid($rows[$rowCount])) continue;
        $vn = $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->selectVar;
        $sql->addSel($vn.' = ', $rows[$rowCount]);
        $rowCount ++;
    }
    for ($i=0; $i < $query->repFilter->size(); $i++){
        $filter = $query->repFilter->getPos($i);
        if ($filter->isJoinedFilter()) continue;
        if ($filter->isValueFilter()) continue;
        if (! $filter->isSet()) continue;
        $sql->addFilterSQL($filter->getName(), $filter, 'base');
    }
    
    if (array_key_exists('colNr', $_REQUEST)){
        $rc = $query->repColumns->getPos($_REQUEST['colNr']);
        if (false) $rc = new RepColumn('');
        // HIER Col-Filter und Col-Wert mit addSel hinzufügen
        error_log('Col-Update '.print_r($_REQUEST,true));     
        $mrc = $query->getReportMetaRecord($rc->getName());
        if (false) $mrc = new MetaRecord();
        $sql->addSel($mrc->getSelectVar().' = ', $_REQUEST['colValue']);
        if (isValid($rc->filter)) $sql->addFilterSQL($rc->filter, $rc, 'base');
    }
}

if (session_status() == PHP_SESSION_NONE)  session_start();

/**
 * 
 * @var Query $query
 */
$query = $_SESSION['WSReportErfassung'] ;
leseRechteVerlangen();
/*
if (! dqAdminRechte()){
    http_response_code(425);
    echo "Für diese Operation sind Administrator-Rechte \nfür die Datenquelle erforderlich";
    die();
}
*/
session_write_close();

if ($query == null){
    http_response_code(425);
    echo 'FEHLER: Datenquelle zur Erfassung nicht gefunden.\nSession-Timeout?';
    die();
}

$varName = $_REQUEST['varName'];
$rows = explode(';', $_REQUEST['row']);

error_log('UpdateValue für '.print_r($_REQUEST,true));

/**
 * 
 * @var MetaRecord $metaRecord
 */
$metaRecord = $query->getReportMetaRecord($varName);

if ($metaRecord == null){
    http_response_code(425);
    echo 'updateValue: '.$varName.' nicht gefunden.';
    die();
}

if (! $metaRecord->enterValue){
    http_response_code(425);
    echo 'updateValue: '.$varName.' ist kein registriertes Eingabefeld.';
    die();
}
if (! isValid($metaRecord->qSetVarname)){
    http_response_code(425);
    echo 'updateValue: Metadaten für Eingabefeld '.$varName." sind unvollständig\nqSetVarname fehlt";
    die();
}

// Wert für Update formatieren

if (isValid($_REQUEST['value'])){
    $vt = $_REQUEST['value'];
    $value = $metaRecord->reformatValue($_REQUEST['value']);
    /**
     * Geldwerte können auch in Tsd. und Mio. eingegeben sein.
     */
    if ($metaRecord->type == MetaRecord::GELD){
        $value = $value / $query->repGeldFaktor;
    }
}else{
    $value = null;
}

createHistory();



$sql = new SqlAbfrage('DWH');
$sql->sql = 'update '.$query->phys_table." base";
foreach ($query->dimensions as $dim){
    $mr = $query->getReportMetaRecord($dim->baseKey);
    if ($mr != null){
        $sql->sql .= "\n join ";
        $sql->sql .= $dim->phys_table.' d'.$dim->dimNr;
        $sql->sql .= ' on '.$dim->dimSelectVar;
        $sql->sql .= ' = '.$mr->getSelectVar();
        if ($dim->histType == Query::VONBIS){
            if (isValid($dim->historyQueryDate)){
                $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$dim->historyQueryDate;
                $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$dim->historyQueryDate;
            }else if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
            }
        }else
            if ($dim->histType == Query::STICHTAG
                || $dim->histType == Query::STICHTAGE){
                    if (isValid($dim->historyQueryDate)){
                        $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$dim->historyQueryDate;
                    } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                        $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                    }
        }
    }else{
        error_log("Dimension $dim->dimNr {$dim->dimName} passt nicht zu {$query->class_table} mit Key {$dim->baseKey}");
    }
}

// UPDATE

if ($metaRecord->type == MetaRecord::STRING
    || $metaRecord->type == MetaRecord::DATE
    || $metaRecord->type == MetaRecord::DATETIME
    || $metaRecord->type == MetaRecord::DATETIMES
    || $metaRecord->type == MetaRecord::TIME
    || $metaRecord->type == MetaRecord::TIMES
    ){
    // Geld-Wert verteilen
    $sql->addKlausel(' set '.$metaRecord->qSetVarname.' =  ',$value );
    error_log('updateValue: Setze Wert  '.$value );    
}
else

if ($value == null){
    // Wert auf NULL setzen
    $sql->sql .= ' set '.$metaRecord->qSetVarname.' = null';
    error_log('updateValue: Setze NULL für Zeilen '.$_REQUEST['row']);
}else
if ($value == 0){
    // Wert auf 0 setzen
    $sql->addKlausel(' set '.$metaRecord->qSetVarname.' = 0');
    error_log('updateValue: Wert 0 für Zeilen '.$_REQUEST['row']);
}
else{
    // numerischen Wert verteilen
    // zuerst aktuellen Wert ermitteln
    $sqlS = new SqlAbfrage('DWH');
    $sqlS->sql = 'select '.$metaRecord->getSelectVar().' as wert, count(*) as Anzahl from '.$query->phys_table ." base\n";
    foreach ($query->dimensions as $dim){
        $mr = $query->getReportMetaRecord($dim->baseKey);
        if ($mr != null){
            $sqlS->sql .= "\n join ";
            $sqlS->sql .= $dim->phys_table.' d'.$dim->dimNr;
            $sqlS->sql .= ' on '.$dim->dimSelectVar;
            $sqlS->sql .= ' = '.$mr->getSelectVar();
            if ($dim->histType == Query::VONBIS){
                if (isValid($dim->historyQueryDate)){
                    $sqlS->sql .= "\n and ".$dim->histAbSelect.' <= '.$dim->historyQueryDate;
                    $sqlS->sql .= "\n and ".$dim->histBisSelect.' >= '.$dim->historyQueryDate;
                }else if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                    $sqlS->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                    $sqlS->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
                }
            }else
                if ($dim->histType == Query::STICHTAG
                    || $dim->histType == Query::STICHTAGE){
                        if (isValid($dim->historyQueryDate)){
                            $sqlS->sql .= "\n and ".$dim->histStichtagSelect.' = '.$dim->historyQueryDate;
                        } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                            $sqlS->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                        }
            }
        }else{
            error_log("Dimension $dim->dimNr {$dim->dimName} passt nicht zu {$query->class_table} mit Key {$dim->baseKey}");
        }
    }
    addSel($sqlS);
    $res = $sqlS->execute();
    if ($res === false){
        http_response_code(425);
        var_dump($sqlS->errorInfo());
        var_dump($sqlS);
        die();
    }
    $z = $res->fetchObject();
    if ($z === false){
        http_response_code(425);
        echo 'Keine passenden Zeilen gefunden';
        error_log ("Keine passenden Zeilen gefunden.\n".$sqlS->sql);
        die();
    }
    
    
    if ($z->wert !==null && $z->wert != 0){
        // numerische Werte multiplizieren
            $sql->addKlausel(' set '.$metaRecord->qSetVarname.' = '.$metaRecord->qSetVarname.' * ',$value / $z->wert);        
            error_log('updateValue: Multipliziere mit  '.($value / $z->wert).' ('.$vt.') für '.$z->Anzahl.' Zeilen '.$_REQUEST['row']);    
    }else 
        // bei NULL und 0
    if ($metaRecord->type == MetaRecord::GELD){
        // Geld-Wert verteilen
        $sql->addKlausel(' set '.$metaRecord->qSetVarname.' =  ',$value / $z->Anzahl);
        error_log('updateValue: Setze Geld-Wert  '.($value / $z->Anzahl).' ('.$vt.') für '.$z->Anzahl.' Zeilen '.$_REQUEST['row']);
    }else{
        // Wert eintragen
        $sql->addKlausel(' set '.$metaRecord->qSetVarname.' = ',$value );
        error_log('updateValue: Setze neuen Wert  '.($value ).' ('.$vt.') für '.$z->Anzahl.' Zeilen '.$_REQUEST['row']);
    }

}
    addSel($sql);
    if (false === $sql->execute()){
        http_response_code(425);
        var_dump($sql->errorInfo());
        var_dump($sql);
        die();
    }

    echo 'OK';
?>